@@ -2,12 +2,19 @@
 
 # Linux build only
 install_linux_deps() {
+	local graphics=1
+	if [[ "$1" == "--headless" ]]; then
+		graphics=
+		shift
+	fi
 	local pkgs=(
 		cmake gettext postgresql
-		libpng-dev libjpeg-dev libgl1-mesa-dev libxi-dev libfreetype-dev
-		libsqlite3-dev libhiredis-dev libogg-dev libgmp-dev libvorbis-dev
-		libopenal-dev libpq-dev libleveldb-dev libcurl4-openssl-dev libzstd-dev
-		libssl-dev
+		libsqlite3-dev libhiredis-dev libogg-dev libgmp-dev libpq-dev
+		libleveldb-dev libcurl4-openssl-dev libzstd-dev libssl-dev
+	)
+	[ -n "$graphics" ] && pkgs+=(
+		libpng-dev libjpeg-dev libgl1-mesa-dev libsdl2-dev libfreetype-dev
+		libogg-dev libvorbis-dev libopenal-dev
 	)
 
 	sudo apt-get update
@@ -29,7 +36,7 @@ install_linux_deps() {
 install_macos_deps() {
 	local pkgs=(
 		cmake gettext freetype gmp jpeg-turbo jsoncpp leveldb
-		libogg libpng libvorbis luajit zstd
+		libogg libpng libvorbis luajit zstd sdl2
 	)
 	export HOMEBREW_NO_INSTALLED_DEPENDENTS_CHECK=1
 	export HOMEBREW_NO_INSTALL_CLEANUP=1
